/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.battlefield;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;

import javax.swing.JPanel;

import fb.battlefieldelements.Figure;
import fb.main.FB;
import fb.util.FBColors;
import fb.util.Resources;

/**
 * panel containing a small representation of the whole battlefield, use to move
 * quickly the view on some area of the battlefield
 * 
 * @author farfpeuf
 * 
 */
public class MiniBattleFieldMapPanel extends JPanel {

	// background image
	protected Image bgImg;

	// image and graphic context used to draw in memory
	// before displaying
	protected Image offimage;

	// map width and length
	private int miniWidth, miniLength;

	// scale factor to compare with battlefield panel
	private float scaleInverse;

	// maximum map width and length
	private static int MAX_WIDTH = 144, MAX_LENGTH = 96;

	// private static int MAX_WIDTH = 50, MAX_LENGTH = 30;

	/**constructor
	 * (initializes tool tip text)
	 */
	public MiniBattleFieldMapPanel() {
		setToolTipText(Resources.getString("minimap.tt"));
	}

	/**
	 * initialize panel
	 * @param mapName map name
	 * @param mapWidth map width in game unit
	 * @param mapLength map length in game unit
	 */
	public void init(String mapName, float mapWidth, float mapLength) {
		float scaleInverseW;
		float scaleInverseL;
		if (FB.getInstance().isMini()) {
			scaleInverseW = mapWidth / 50;
			scaleInverseL = mapLength / 30;
		} else {
			scaleInverseW = mapWidth / MAX_WIDTH;
			scaleInverseL = mapLength / MAX_LENGTH;
		}

		if (scaleInverseW > scaleInverseL) {
			scaleInverse = scaleInverseW;
		} else {
			scaleInverse = scaleInverseL;
		}
		miniWidth = (int) (mapWidth / scaleInverse);
		miniLength = (int) (mapLength / scaleInverse);
		Dimension d = new Dimension(miniWidth, miniLength);
		setSize(d);
		setPreferredSize(d);
		setMinimumSize(d);
		setBackground(FBColors.background);
		bgImg = MiniMapPainter.getPaintedMap(mapName, true).getImage();
		MediaTracker tracker = new MediaTracker(this);
		tracker.addImage(bgImg, 0);
		try {
			tracker.waitForAll();
		} catch (InterruptedException e) {
			FB.exception(e);
		}
	}

	/**
	 * overridden to draw map image and warriors
	 */
	public void paintComponent(Graphics graf) {

		super.paintComponent(graf);
		graf.drawImage(bgImg, 0, 0, miniWidth, miniLength, null);
		Figure model;
		for (int i = 0; i < BattleFieldModel.getInstance().getNbWarriors(); i++) {
			model = BattleFieldModel.getInstance().getWarrior(i);
			model.minipaint(graf, scaleInverse);
		}

	}

	/**
	 * @return Returns the length.
	 */
	public int getMiniLength() {
		return miniLength;
	}

	/**
	 * @return Returns the width.
	 */
	public int getWidth() {
		return miniWidth;
	}

}