/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.battlefield;

import java.awt.Color;

/**
 * @author Farfpeuf
 * interface implemented by all elements on the battlefield
 * painted using a sprite
 */
public interface PaintedWithSprite {
	/**
	 * define available sequences for the sprite
	 * @param sprite the sprite
	 */
	public void addSpriteSequences(Sprite sprite);
	/**
	 * return the sprite id
	 * @return the sprite id
	 */
	public int getSpriteID();
	/**
	 * return number of frames in the sprite
	 * @return the number of frames
	 */
	public int getNumberOfFrames();
	/**
	 * get the sprite used for painting
	 * @return the sprite
	 */
	public Sprite getSprite();
	/**
	 * get width
	 * @return width
	 */
	public float getWidth();
	/**
	 * get length
	 * @return  length
	 */
	public float getLength();
	/**
	 * get scale
	 * @return  scale
	 */
	public float getScale();
	/**
	 * get step size
	 * @return distance for each step
	 */
	public int getStep();
	/**
	 * get army color
	 * @return the army color
	 */
	public Color getColor();
}
