/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;

import java.awt.Point;
import java.awt.Rectangle;

import fb.army.hierarchy.Army;
import fb.artificialintelligence.AStar;
import fb.main.FBConstants;

/**
 * find a path in campaign map using a-star algorythm
 * 
 * @author farfpeuf
 * 
 */
public class CampaignPathFinder extends AStar {

	/**
	 * army for which path is searched
	 */
	protected Army army;

	/**
	 * map bounds
	 */
	protected Rectangle bounds;

	/**
	 * constructor
	 * 
	 * @param origin
	 *            origin point
	 * @param destination
	 *            destination point
	 * @param army
	 *            army for which path is searched
	 * @param bounds
	 *            map bounds
	 */
	public CampaignPathFinder(Point origin, Point destination, Army army,
			Rectangle bounds) {
		super(origin, destination);
		this.army = army;
		this.bounds = bounds;
	}

	/**
	 * find point surrounding a location
	 * 
	 * @param currentPosition
	 *            the current point
	 * @return the surrounding points
	 */
	public Point[] getSurroundingPoints(Point currentPosition) {
		Point[] result = new Point[6];
		for (int direction = 0; direction < 6; direction++) {
			result[direction] = Hexagon.getTranslatedCoords(currentPosition, 1,
					direction);
		}
		return result;
	}

	/**
	 * is this position legal
	 * 
	 * @param position
	 *            the position to test
	 * @return true if position is legal
	 */
	public boolean isLegalPosition(Point position) {
		return bounds.contains(position);
	}

	/**
	 * calculate cost from parent node
	 * 
	 * @param parent
	 *            the parent position
	 * @param position
	 *            the child position
	 * @return the cost
	 */
	public int costFromParent(Point parent, Point position) {
		char tile = HexagonalMapModel.getInstance().getTile(parent.x, parent.y);

		int terrainType = 2;
		switch (tile) {
		case 'e':
			terrainType = 0;
			break;
		case 'b':
			terrainType = 1;
			break;
		case 'a':
			terrainType = 2;
			break;
		case 'h':
			terrainType = 3;
			break;
		case 'd':
			terrainType = 4;
			break;
		}

		return FBConstants.TERRAIN_COST[terrainType][army.getRaceId()];
	}

	/**
	 * estimate cost to destination
	 * 
	 * @param position
	 *            the current position
	 * @param destination
	 *            the destination point
	 * @return the estimated cost
	 */
	public int estimateCostToDestination(Point position, Point destination) {
		return Hexagon.getDirectDistance(position, destination) * 4;
	}

}
