/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextArea;

import fb.main.FB;
import fb.util.FBColors;
import fb.util.GraphicTools;

/**
 * frame displayed when loading game resources
 * 
 * @author Farfpeuf
 */
public class WaitFrame extends JPanel {

	/**
	 * singleton instance
	 */
	private static WaitFrame instance;

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static  synchronized WaitFrame getInstance() {
		if (null == instance) {
			instance = new WaitFrame();
		}
		return instance;
	}

	/**
	 * delete singleton to free memory
	 * 
	 */
	public static synchronized void freeInstance() {
		if (null != instance) {
			instance = null;
		}
	}

	/**
	 * loading progress bar
	 */
	private JProgressBar progressBar = new JProgressBar(0, 100);

	/**
	 * text area to display witing message
	 */
	private JTextArea waitarea = new JTextArea(10, 10);

	/**
	 * constructor
	 * 
	 * build and assemble elements
	 * 
	 */
	public WaitFrame() {
		setLayout(new GridBagLayout());

		setOpaque(false);
		progressBar.setStringPainted(true);

		waitarea.setBackground(FBColors.background);
		waitarea.setForeground(FBColors.standardText);

		GridBagConstraints gbc = GraphicTools.createStandardConstraint();
		gbc.fill = GridBagConstraints.BOTH;
		add(progressBar, gbc);
		gbc.gridy++;
		add(waitarea, gbc);
		setBounds((int) (0 * FB.getXres()), (int) (0 * FB.getYres()),
				(int) (800 * FB.getXres()), (int) (650 * FB.getYres()));
	}

	/**
	 * change progression text
	 * 
	 * @param text
	 *            new progression text
	 */
	public static void setProgressionText(String text) {
		getInstance().progressBar.setString(text);
	}

	/**
	 * change progression value
	 * 
	 * @param text
	 *            new progression value
	 */
	public static void setProgressionValue(int value) {
		getInstance().progressBar.setValue(value);
	}

	/**
	 * append a new message in text area
	 * 
	 * @param text
	 *            the new message
	 */
	public static void message(String text) {
		getInstance().waitarea.append(text);
	}

}
