/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import fb.army.entities.Entity;
import fb.army.hierarchy.Army;
import fb.gui.frames.BattleFrame;
import fb.gui.widgets.armytreetable.TreeTable;
import fb.gui.widgets.armytreetable.TreeTableModelAdapter;
import fb.main.FB;
import fb.main.Battle;
import fb.phases.Phase;
import fb.util.Resources;

/**
 * displays current player, time and phase and armies, report, help and chat
 * tabs
 * 
 * @author Farfpeuf
 */
public class BattleStatePanel extends StatePanel {
	/**
	 * singleton instance
	 */
	private static BattleStatePanel instance;

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static synchronized BattleStatePanel getInstance() {
		if (null == instance) {
			instance = new BattleStatePanel();
		}
		return instance;
	}

	/**
	 * build all the tabs (armies, report, help and chat)
	 */
	public void buildTabs() {
		super.buildTabs();
		Army army;
		TreeTablePanel armyTreePanel;
		for (int i = 0; i < Battle.getInstance().getArmies().length; i++) {
			army = Battle.getInstance().getArmies()[i];
			armyTreePanel = new TreeTablePanel(army, true);
			armyTreePanel.getTreeTable().setColumnWidths(
					new int[] { 300, 80, 100 });
			armyTreePanel.setToolTipText(Resources.getString("playetree.tt"));
			armyTreePanel.getTreeTable().setToolTipText(
					Resources.getString("armytree.tt"));
			tabbedPane.addTab(army.getPlayerName(), armyTreePanel);
			if (FB.getInstance().isGraphicMode()) {
				final TreeTable table = armyTreePanel.getTreeTable();

				table.getSelectionModel().addListSelectionListener(
						new ListSelectionListener() {
							public void valueChanged(ListSelectionEvent e) {
								ListSelectionModel model = (ListSelectionModel) e
										.getSource();

								TreeTableModelAdapter ad = (TreeTableModelAdapter) table
										.getModel();
								Object leadNode = ad.nodeForRow(model
										.getLeadSelectionIndex());

								FB.getInstance().centerOn(leadNode);
								if (leadNode instanceof Entity) {
									BattleFrame.getInstance()
											.displayEntityCharacteristics(
													(Entity) leadNode);
								}
							}
						});
			}
		}

	}

	/**
	 * change current phase
	 * 
	 * @param phase
	 *            the new phase
	 */
	public void setPhase(Phase phase) {
		super.setPhase(Resources.getPhaseIcon(phase.getIconNumber()));
	}

	/**
	 * change current time
	 */
	public void setTime(int time) {
		timelabel.setIcon(Resources.getTimeIcon(time - 1));
	}

	/**
	 * change current player
	 * 
	 * @param army
	 *            the player army
	 */
	public void setPlayer(Army army) {
		if (null == army) {
			playerLabel.setText("");
			playerLabel.setBorder(null);
		} else {
			setPlayer(army.getColor(), army.getPlayerName(), army.isAi(), army
					.isHandled());
		}

	}

}
