/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fb.battlefield.MapModel;
import fb.battlefield.MiniMapPainter;
import fb.gui.widgets.FBComboBox;
import fb.gui.widgets.SelectableListener;
import fb.main.FB;
import fb.network.Client;
import fb.network.Server;
import fb.util.FBColors;
import fb.util.FileTools;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * panel to choose a battle map
 * 
 * @author farfpeuf
 * 
 */
public class MapChooser extends JPanel {
	/**
	 * singleton editable and non editable instance
	 */
	private static MapChooser editableInstance, nonEditableInstance;

	/**
	 * current selected map name
	 */
	private String mapName;

	/**
	 * available maps list
	 */
	private List maplist;
	{
		maplist = FileTools.list("mapmodels");
	}

	/**
	 * combo to choose map
	 */
	private FBComboBox mapCombo;

	/**
	 * map name label
	 */
	private JLabel mapNameLabel;

	/**
	 * map label
	 */
	private JLabel mapLabel;

	/**
	 * get singleton instance
	 * 
	 * @return editable or non editable instance
	 */
	public static synchronized MapChooser getInstance() {
		return getInstance(Server.hasInstance() || !FB.isNet());
	}

	/**
	 * get singleton instance
	 * 
	 * @param editableMode
	 *            is map chosen by this computer or not ?
	 * @return editable or non editable instance
	 */
	public static synchronized MapChooser getInstance(boolean editableMode) {
		if (editableMode) {
			if (null == editableInstance) {
				editableInstance = new MapChooser(editableMode);
			}
			return editableInstance;
		} else {
			if (null == nonEditableInstance) {
				nonEditableInstance = new MapChooser(editableMode);
			}
			return nonEditableInstance;
		}
	}

	/**
	 * constructor build and assemble panel elements
	 * 
	 * @param editableMode
	 *            is map chosen by this computer or not ?
	 */
	private MapChooser(boolean editableMode) {
		setLayout(new PercentageLayout());
		setBackground(FBColors.background);

		mapLabel = new JLabel();
		if (editableMode) {
			mapCombo = new FBComboBox(maplist.toArray());
			mapChanged();
			mapCombo.addItemListener(new SelectableListener() {
				public void threadedItemStateChanged() {
					mapChanged();
				}
			});
			add(mapCombo, new PercentageLayout.Constraint(20, 80, 60, 20));
		} else {
			mapNameLabel = new JLabel();
			mapNameLabel.setOpaque(false);
			mapNameLabel.setForeground(FBColors.primary);
			mapNameLabel.setHorizontalAlignment(JLabel.CENTER);
			add(mapNameLabel, new PercentageLayout.Constraint(20, 80, 60, 20));
		}

		mapLabel.setHorizontalAlignment(0);

		add(mapLabel, new PercentageLayout.Constraint(1, 1, 98, 78));
		setBorder(BorderFactory.createLineBorder(FBColors.darkBorder));
		validate();
		repaint();
		setToolTipText(Resources.getString("field.tt"));
	}

	/**
	 * handle map change
	 * 
	 */
	private void mapChanged() {
		setMap((String) mapCombo.getSelectedItem());
		if (FB.isNet()) {
			Client.getInstance().send("SETMAP" + mapName);
		}
	}

	/**
	 * get selected map name
	 * 
	 * @return the selected map name
	 */
	public String getSelectedMap() {
		return mapName;
	}

	/**
	 * change the map
	 * 
	 * @param newName
	 *            the new map
	 */
	public void setMap(String newName) {
		if (!newName.equals(mapName)) {
			Resources.free(Resources.getMapKey(mapName, MapModel.getInstance()
					.isFlipLR(), MapModel.getInstance().isFlipUD()));
			mapName = newName;
			if (!FB.getInstance().isBatch()) {
				mapLabel.setIcon(MiniMapPainter.getPaintedMap(mapName, false));
			}
		}
	}

}