/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.main;

import fb.gui.frames.BattleFrame;
import fb.network.Client;
import fb.network.MessagesConstants;
import fb.phases.AIMagicPhase;
import fb.phases.AIOrderPhase;
import fb.phases.AutomaticMovementPhase;
import fb.phases.BinaryPhase;
import fb.phases.CombatPhase;
import fb.phases.DeploymentPhase;
import fb.phases.Phase;
import fb.phases.InterruptingPhase;
import fb.phases.MovementPhase;
import fb.phases.ShootingPhase;
import fb.phases.SynchronizedPhase;
import fb.phases.ThreatPhase;
import fb.util.StringTools;

/**
 * battle main loop
 * 
 * @author farfpeuf
 * 
 */
public class BattleLoop {
	/**
	 * waiting for action ?
	 */
	private boolean waiting = false;

	/**
	 * the current game phase
	 */
	private Phase currentPhase;

	/**
	 * armies that finished current phase
	 */
	private boolean[] phaseTokens = new boolean[0];

	/**
	 * number of armies that finished current phase
	 */
	private int numberOfPhaseTokens = 0;

	/**
	 * current synchronized phase
	 */
	public SynchronizedPhase currentSynchronizedPhase;

	/**
	 * number of loop interruptions
	 */
	public int interrupted = 0;

	/**
	 * stop loop ?
	 */
	private boolean stop = false;

	/**
	 * reset loop
	 * 
	 */
	public void reset() {
		init();
		numberOfPhaseTokens = 0;
		phaseTokens = new boolean[0];
	}

	/**
	 * initialize loop
	 * 
	 */
	public void init() {
		currentSynchronizedPhase = null;
		currentPhase = null;
		interrupted = 0;
		stop = false;
	}

	/**
	 * reset phase tokens array
	 * 
	 */
	public synchronized void resetPhaseTokens() {
		phaseTokens = new boolean[Battle.getInstance().getArmies().length];
		numberOfPhaseTokens = 0;
	}

	/**
	 * @return number Of PhaseTokens
	 */
	public synchronized int getNumberOfPhaseTokens() {
		return numberOfPhaseTokens;
	}

	/**
	 * read phase tokens from string buffer read from a file
	 * 
	 * @param buffy
	 *            the sting buffer
	 */
	public void loadTokens(StringBuffer buffy) {
		int size = StringTools.readBufferInt(buffy);
		phaseTokens = new boolean[size];
		for (int i = 0; i < size; i++) {
			phaseTokens[i] = (StringTools.readBufferChar(buffy) == 'y');
			if (phaseTokens[i]) {
				numberOfPhaseTokens++;
			}
		}
	}

	/**
	 * @return phase tokens as a string to save to a file
	 */
	public String saveTokens() {
		String result = "";
		result += phaseTokens.length;
		for (int i = 0; i < phaseTokens.length; i++) {
			result += "\t" + (phaseTokens[i] ? 'y' : 'n');
		}
		return result;
	}

	/**
	 * do set token for specified army
	 * 
	 * @param armyId
	 *            the army id
	 */
	public synchronized void doSetToken(int armyId) {
		if (!phaseTokens[armyId]) {
			phaseTokens[armyId] = true;
			numberOfPhaseTokens++;
			if (!Battle.getInstance().getArmies()[armyId].isHandled()) {
				resume();
			}
		} else {
			FB.exception(new Exception("DOUBLE DONE"));
			if (FB.getInstance().isDebug()) {

				System.exit(-1);
			}
		}
	}

	/**
	 * @return phase tokens array
	 */
	public synchronized boolean[] getPhaseTokens() {
		return phaseTokens;
	}

	/**
	 * get the next phase
	 * 
	 * @param phase
	 *            the current phase
	 * @return the next phase
	 */
	private static Phase getNextPhase(Phase phase) {
		Phase newPhase = null;

		if (phase instanceof CombatPhase) {
			Battle.getInstance().timePlus();
		}
		if (phase == null) {
			newPhase = new DeploymentPhase();

		} else if (phase == null || phase instanceof DeploymentPhase
				|| phase instanceof CombatPhase) {
			newPhase = new ShootingPhase();

		} else if (phase instanceof AIMagicPhase) {
			newPhase = new ThreatPhase();
		} else if (phase instanceof InterruptingPhase) {
			newPhase = ((InterruptingPhase) phase).getInterrupted();
		} else if (phase instanceof ShootingPhase) {
			newPhase = new AIMagicPhase();

		} else if (phase instanceof ThreatPhase) {
			newPhase = new AutomaticMovementPhase();

		} else if (phase instanceof AutomaticMovementPhase) {
			newPhase = new MovementPhase();

		} else if (phase instanceof MovementPhase) {
			newPhase = new AIOrderPhase();

		} else if (phase instanceof AIOrderPhase) {
			newPhase = new CombatPhase();

		}
		return newPhase;
	}

	/**
	 * change current phase
	 * 
	 */
	private void changePhase() {
		boolean interuption = false;
		if (currentPhase != null && currentPhase instanceof InterruptingPhase) {
			interuption = true;
		} else {
			synchronizeWithOtherClients();
		}
		currentPhase = getNextPhase(currentPhase);

		if (currentPhase instanceof SynchronizedPhase) {
			currentSynchronizedPhase = (SynchronizedPhase) currentPhase;
		}
		if (interuption) {
			((BinaryPhase) currentPhase).resume();
		} else {
			resetPhaseTokens();
			synchronizeWithOtherClients();
			currentPhase.init();
		}
	}

	/**
	 * interrupt current phase with another phase
	 * 
	 * @param playPhase
	 *            the other phase
	 */
	public void interruptWith(InterruptingPhase playPhase) {
		currentPhase = currentSynchronizedPhase = playPhase;
		playPhase.init();
		playPhase.nextStep();
		resume();
	}

	/**
	 * resume loop
	 * 
	 */
	public synchronized void resume() {
		if (waiting) {
			notify();
		}
	}

	/**
	 * pause loop
	 * 
	 * @throws InterruptedException
	 */
	public synchronized void pause() throws InterruptedException {
		waiting = true;
		wait();
		waiting = false;
	}

	/**
	 * start loop
	 * 
	 */
	public void start() {
		Thread thread = new Thread("main loop") {
			public void run() {
				try {
					if (currentPhase != null) {
						new DeploymentPhase().calculateResults();
						BattleFrame.getInstance().enableButtons(true);
					} else {
						changePhase();
					}
					runLoop();
				} catch (Exception e) {
					FB.exception(e);
				}
			}
		};
		thread.start();

	}

	/**
	 * loop actions
	 * 
	 */
	private void runLoop() {
		try {

			while (!stopped()) {
				runPhase();
				synchronized (this) {
					if (mustWait()) {
						pause();
					}
				}

				if (currentPhase.finished()) {
					currentPhase.end();
					changePhase();
				}

				if (battleFinished()) {
					endBattle();
				}
			}

		} catch (Exception e) {
			FB.exception(e);
		}
	}

	/**
	 * @return true if loop is waiting for player or net action
	 */
	private boolean mustWait() {

		return currentPhase.isWaitingForPlayerAction()
				|| currentPhase.isWaitingForNetAction();
	}

	/**
	 * run current phase
	 * 
	 */
	private void runPhase() {
		while (!stopped() && !mustWait() && !currentPhase.finished()) {
			currentPhase.nextStep();
		}
	}

	/**
	 * end the battle
	 * 
	 */
	private void endBattle() {
		int winner = Battle.getInstance().winner();
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(MessagesConstants.ENDGAME + winner);
		} else {
			doEndBattle(winner);
		}
	}

	/**
	 * do end the battle
	 * 
	 * @param winner
	 *            id of winning army
	 */
	public void doEndBattle(int winner) {
		if (!stopped()) {
			stop();
			Battle.getInstance().end(winner);
		}
	}

	/**
	 * synchronize loop with other clients
	 * 
	 */
	private void synchronizeWithOtherClients() {
		if (FB.isNet()) {
			Client.getInstance().synchronize();
		}
	}

	/**
	 * notify end of interruption and resume loop
	 * 
	 */
	public synchronized void notifyAndResume() {
		interrupted--;
		resume();
	}

	/**
	 * interrupt current phase
	 * 
	 * @return the interrupted phase
	 */
	public synchronized BinaryPhase interruptCurrentPhase() {
		interrupted++;
		return ((BinaryPhase) currentSynchronizedPhase);

	}

	/**
	 * 
	 * stop loop
	 * 
	 */
	public void stop() {
		stop = true;
		if (currentSynchronizedPhase != null) {
			currentSynchronizedPhase.stop();
			currentSynchronizedPhase = null;
		}
		currentPhase = null;
	}

	/**
	 * @return true if loop was topped
	 */
	public boolean stopped() {
		return stop;
	}

	/**
	 * @return true if the battle is finished
	 */
	private boolean battleFinished() {
		return Battle.getInstance().winner() != -1;
	}

	/**
	 * set the phase tokens
	 * 
	 * @param tokens
	 *            the phase tokens
	 */
	public void setPhaseTokens(boolean[] tokens) {
		this.phaseTokens = tokens;

	}

	/**
	 * set number of phase tokens
	 * 
	 * @param numberOfPhaseTokens
	 *            the number of phase tokens
	 */
	public void setNumberOfPhaseTokens(int numberOfPhaseTokens) {
		this.numberOfPhaseTokens = numberOfPhaseTokens;
	}

	/**
	 * @return current Synchronized Phase
	 */
	public SynchronizedPhase getCurrentSynchronizedPhase() {
		return currentSynchronizedPhase;
	}

	/**
	 * change current Synchronized Phase
	 * 
	 * @param currentSynchronizedPhase
	 *            the new Synchronized Phase
	 */
	public void setCurrentSynchronizedPhase(
			SynchronizedPhase currentSynchronizedPhase) {
		this.currentSynchronizedPhase = currentSynchronizedPhase;
		currentPhase = currentSynchronizedPhase;
	}

	/**
	 * @return the current Phase
	 */
	public Phase getCurrentPhase() {
		return currentPhase;
	}
}
