/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import java.util.List;

import fb.army.entities.Courier;
import fb.army.entities.Entity;
import fb.army.entities.Leader;
import fb.army.entities.MessageReceiver;
import fb.army.entities.Unit;
import fb.army.hierarchy.Army;
import fb.artificialintelligence.TargetFinder;
import fb.main.FB;
import fb.main.Battle;
import fb.util.D6;
import fb.util.DX;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * order phase for computer players
 * 
 * @author farfpeuf
 * 
 */
public class AIOrderPhase extends BinaryPhase {

	/**
	 * current step is order selection ?
	 */
	private boolean orderselection;

	/**
	 * selected new order
	 */
	private char neworder;

	/**
	 * token used to know wich armies finished this phase
	 */
	private boolean[] token = new boolean[Battle.getInstance().getArmies().length];

	/**
	 * number of armies that finished this phase
	 */
	private int tokens = 0;

	/**
	 * phase name
	 */
	private static final String name = "iaorderphase";

	/**
	 * phase icon number
	 */
	private static final int iconNumber = 4;

	/**
	 * @return phase name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return phase icon number
	 */
	public int getIconNumber() {
		return iconNumber;
	}

	/**
	 * @return phase toolip
	 */
	public String getTooltip() {
		return null;// no tooltip
	}

	/**
	 * change current playing army
	 * 
	 */
	public void changeArmy() {
		int i = current + 1;
		if (i == armies.length) {
			i = 0;
		}
		while (token[i] && !finished()) {
			i++;
			if (i == armies.length) {
				i = 0;
			}

		}
		current = i;
		// FB.debug("changeArmy " + armies[current].getPlayerName());
	}

	/**
	 * change current playing army and initialize actors selection
	 */
	public void nextArmy() {
		changeArmy();
		if (armies[current].isAi() && !finished()) {
			initActorSelection();
		}
	}

	/**
	 * proceed to phase next step
	 */

	public void nextStep() {

		FB.setWaitCursor();
		if (armies[current].isAi() && armies[current].isHandled()) {
			handleAI();
		} else {
			token[current] = true;
			tokens++;
			if (!finished()) {
				nextArmy();
			}
		}

	}

	/**
	 * has this entity targets for this phase ?
	 * 
	 * @param actor
	 *            the phase actor
	 * @return true if this entity has targets for this phase ?
	 */
	public boolean hasTargets(Entity actor) {
		Entity entity;
		List entities = armies[current].getActiveEntities();
		for (int i = 0; i < entities.size(); i++) {
			entity = (Entity) (entities.get(i));
			if (entity.isActive()) {
				if (isTarget(actor, entity)) {
					return true;
				}
			}
		}
		return false;

	}

	/**
	 * calculate available targets
	 * 
	 */
	public void calculatePotentialTargets() {

		Entity entity;
		clearPotentialTargets();
		List entities = armies[current].getActiveEntities();
		for (int i = 0; i < entities.size(); i++) {
			entity = (Entity) (entities.get(i));
			if (entity.isActive()) {
				if (isTarget(entity)) {
					addPotentialTarget(entity);
				}
			}

		}
		setActorselection(false);

	}

	/**
	 * @return AI thread that handles choice of the best target among a set of
	 *         availables targets
	 */
	public TargetFinder getTargetFinder() {
		return null;// not used

	}

	/**
	 * calculate next target for computer player
	 * 
	 * @return the target
	 */
	public Entity getAITarget() {
		return (Entity) getPotentialTargets().get(
				DX.roll(getPotentialTargets().size() - 1));
	}

	/**
	 * choose order for computer player
	 * 
	 * @return chosen order
	 */
	public char getAIOrder() {
		int offensive = D6.roll() + D6.roll() + D6.roll();

		if (getActor().suffersBadLight()) {
			offensive -= 3;
		}

		if (getActor().getType() == 'c') {
			offensive += 1;
		}

		if (getActor().getType() == 'a') {
			offensive += 3;
		}

		if (getActor().getType() == 'f') {
			offensive += 4;
		}

		if (armies[current].degradation() > 3) {
			offensive += 2;
		}
		for (int i = 0; i < armies[current].getEnnemies().size(); i++) {
			if (armies[current].degradation() > ((Army) armies[current]
					.getEnnemies().get(i)).degradation() + 2) {
				offensive -= 3;
			}

			if (armies[current].degradation() + 2 < ((Army) armies[current]
					.getEnnemies().get(i)).degradation()) {
				offensive += 3;
			}
		}
		if (offensive > 12) {
			return 'a';
		} else if (offensive > 7) {
			return 'o';
		} else {
			return 'h';
		}

	}

	/**
	 * handle computer player
	 * 
	 */
	public void handleAI() {

		if (isActorSelection()) {

			setActor((Entity) getPotentialActors().get(
					DX.roll(getPotentialActors().size() - 1)));
			setActorselection(false);
			clearPotentialTargets();
			doneEntities.add(getActor());

		} else if (!orderselection) {

			orderselection = true;

			if (getActor().isCourier()) {
				initTargetSelection();
				setTarget(getAITarget());
			}

		} else {

			orderselection = false;
			neworder = getAIOrder();

			if (neworder != getActor().getCommand().getOrder()) {
				action();
			}
			nextArmy();
		}
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for current actor
	 */
	public boolean isTarget(Entity e) {
		if (e.isLeader() && !e.isWarchief() && e.getArmy() == armies[current]) {
			return true;
		}
		return false;
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            phase actor
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity actor, Entity e) {
		return true;
	}

	/**
	 * is this entity a potential actor for this phase
	 * 
	 * @param a
	 *            phase actor
	 * @return true if this entity is a potential actor
	 */
	public boolean isActor(Entity e) {
		if (e.isMessageReceiver() && e.getArmy() == armies[current]) {
			MessageReceiver r = (MessageReceiver) e;

			if (r.getNewOrder() == 'n') {
				if (r.isWarchief()) {
					Leader l = (Leader) r;
					if (l.orderDistanceOkForAll()
							&& l.getCommand().getUnits().size() != 0) {
						return true;
					}
				} else if (r.nearWarchief()
						&& (r.getCommand().getUnits().size() != 0 || r
								.isCourier())) {
					return true;
				}
			}

		}
		return false;
	}

	/**
	 * proceed to phase action
	 * 
	 */
	public void action() {

		if (!getActor().isCourier()) {
			Leader leader = (Leader) getActor();
			leader.setNewOrder(neworder);
			leader.setMessenger((Leader) armies[current].getWarchief());
		} else { // choix du destinataire du messager
			Courier courier = (Courier) getActor();
			courier.setNewOrder(neworder);
			courier.setReceiver(getTarget());

			String orderstring;
			if (neworder == 'a') {
				orderstring = "attack";
			} else if (neworder == 'o') {
				orderstring = "oppose";
			} else {
				orderstring = "hold";
			}
			String message = Resources.getString("couriernew");
			message = StringTools.replaceAll(StringTools.replaceAll(message,
					"<entity full name>", courier.getFullName()),
					"<order name>", Resources.getString(orderstring));
			FB.getInstance().message(message, courier.getColor());
		}

	}

	/**
	 * calculate phase results
	 * 
	 */
	public void calculateResults() {

		Entity entity;

		for (int i = 0; i < Battle.getInstance().getActiveEntities().size(); i++) {
			entity = (Entity) (Battle.getInstance().getActiveEntities().get(i));
			if (entity.isCourier()) {
				Courier courier = (Courier) entity;
				if (courier.getNewOrder() != 'n') {
					if (courier.near(courier.getReceiver())) {
						Entity receiver = courier.getReceiver();
						if (receiver.isLeader()) {
							((Leader) receiver).setNewOrder(courier
									.getNewOrder());
							((Leader) receiver).setMessenger(courier);
						} else {
							((Unit) receiver)
									.changeOrder(courier.getNewOrder());
							courier.setReceiver(null);
						}

						String message = Resources.getString("couriertransmit");
						message = StringTools.replaceAll(message,
								"<entity full name>", courier.getFullName());
						FB.getInstance().message(message, courier.getColor());
					}

				}
			} // couriers
			else if (entity.isLeader()) {
				Leader leader = (Leader) entity;
				if (leader.getNewOrder() != 'n') {
					if (leader.orderDistanceOkForAll()) {
						if (leader.isWarchief()) {
							leader.getCommand().changeOrder(
									leader.getNewOrder());
						} else if (leader.accceptNewOrders()) {
							if (leader.getMessenger().isCourier()) {
								Courier courier = (Courier) leader
										.getMessenger();
								courier.setNewOrder('n');
								courier.setReceiver(null);
							}

						}
					} else {
						String message = Resources.getString("toofar");
						message = StringTools.replaceAll(message,
								"<entity full name>", leader.getFullName());
						FB.getInstance().message(message,
								leader.getArmy().getColor());
					}
					leader.setNewOrder('n');
					leader.setMessenger(null);
				}
			} // leaders

		}

	}

	/**
	 * set token for specified army
	 */
	public void setToken(int armyId) {
		token[armyId] = true;
		tokens++;
	}

	/**
	 * @return true if all the armies handled by this computer finished this
	 *         phase
	 */
	public boolean allHandledArmiesDone() {
		return tokens == armies.length;
	}

	/**
	 * @return true if phase is finished
	 */
	public boolean finished() {
		return tokens == armies.length;
	}

	/**
	 * phase waiting for network action ?
	 */
	public boolean isWaitingForNetAction() {
		return false;
	}

}
