/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import java.util.Iterator;

import fb.army.entities.Entity;
import fb.army.entities.Shootingunit;
import fb.army.entities.Unit;
import fb.army.hierarchy.Army;
import fb.artificialintelligence.TargetFinder;
import fb.battlefield.BattleFieldModel;
import fb.battlefield.BattleFieldPanel;
import fb.main.FB;
import fb.main.Battle;
import fb.network.Client;

/**
 * projectile units game phase
 * 
 * @author farfpeuf
 * 
 */
public class ShootingPhase extends BinaryPhase {

	/**
	 * phase name
	 */
	private static final String name = "shootingphase";

	/**
	 * phase icon number
	 */
	private static final int iconNumber = 1;

	/**
	 * constructor
	 * 
	 */
	public ShootingPhase() {
		super();
		setWithPhasePanel(true);
	}

	/**
	 * @return phase name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return phase icon number
	 */
	public int getIconNumber() {
		return iconNumber;
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            phase actor
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity actor, Entity e) {
		if (e.getProjectileShield() == 0
				&& (actor.lineOfSightTo(e) < ((Shootingunit) actor)
						.getLongRange() * 8)) {
			return true;
		}
		return false;
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for current actor
	 */
	public boolean isTarget(Entity e) {
		if (e.getProjectileShield() == 0
				&& (getActor().lineOfSightTo(e) < ((Shootingunit) getActor())
						.getLongRange() * 8)) {
			return true;
		}
		return false;
	}

	/**
	 * is this entity a potential actor for this phase
	 * 
	 * @param a
	 *            phase actor
	 * @return true if this entity is a potential actor
	 */
	public boolean isActor(Entity e) {
		if (e.isShootingUnit()) {
			if (e.getOpposingContacts().size() == 0) {
				if (((Shootingunit) e).getProjectilesNumber() > 0) {
					return true;
				}
			}
		}
		return false;

	}

	/**
	 * calculate phase results
	 * 
	 */
	public void calculateResults() {

		for (int i = 0; i < actors.size(); i++) {
			Shootingunit su = (Shootingunit) actors.get(i);
			if (su.isHandled()) {
				Entity tar = (Entity) targets.get(i);
				su.shoot(tar);
			}
		}

		if (FB.isNet()) {
			Client.getInstance().synchronize();
		}

		if (FB.getInstance().isGraphicMode()) {

			for (int i = 0; i < actors.size(); i++) {
				((Shootingunit) actors.get(i)).animShoot((Entity) targets
						.get(i));
			}
		}

		for (int i = 0; i < actors.size(); i++) {

			while (!stop && ((Shootingunit) actors.get(i)).isShooting()) {
				sleep(100);
			}
		}

		while (!stop && BattleFieldModel.getInstance().getNbProjectiles() != 0) {
			sleep(100);

		}

		for (int i = 0; !stop && i < targets.size(); i++) {
			Entity e = (Entity) (targets.get(i));
			while (e.isDying()) {
				sleep(100);
			}
			e.commitKill();
			if (e.getType() != 'u' && e.getState() != 'a') {
				if (e.isUnit() && e.getArmy().isHandled()) {
					Unit u = (Unit) e;
					if (u.getCasualtyPercentage() > 25.0) {
						u.casualtyTest();
					}
				}
			}

		} // for

		if (FB.isNet()) {
			Client.getInstance().synchronize();
		}

		Army[] armies = Battle.getInstance().getArmies();
		for (int i = 0; i < armies.length; i++) {
			if (armies[i].isHandled() && armies[i].mustTestMorale()) {
				armies[i].moraleTest();
			}
		}
		Iterator iter = Battle.getInstance().getActiveEntities().iterator();
		Entity entity;
		while (iter.hasNext()) {
			entity = (Entity) iter.next();
			if (entity.getProjectileShield() > 0) {
				entity.consumeProjectileShield();
			}
		}

		for (int i = 0; i < actors.size(); i++) {
			Shootingunit su = (Shootingunit) actors.get(i);
			su.spendVolleys();
		}

	}

	/**
	 * @return AI thread that handles choice of the best target among a set of
	 *         availables targets
	 */
	public TargetFinder getTargetFinder() {
		int maxWeight = 17;
		Shootingunit su = (Shootingunit) getActor();

		if (su.getProjectilesNumber() > 1) {
			maxWeight -= 1;
		}

		if (su.getProjectilesNumber() > 4) {
			maxWeight -= 2;
		}

		return new TargetFinder(getPotentialTargets(), 3, maxWeight, getActor()) {

			protected int getWeight(Object element) {
				Entity potentialTarget = (Entity) element;
				int poid = super.getWeight(potentialTarget);

				Shootingunit su = (Shootingunit) entity;

				poid +=  (12 - su.shootToHit(potentialTarget)/7);

				poid += (12 - su.shootToKill(potentialTarget)/7);

				if (potentialTarget == getClosestTarget()) { // elem
					poid += 2;
				}
				if (potentialTarget.surface() > 1) { // elem
					poid += 1;
				}
				if (potentialTarget.surface() > 2) { // elem
					poid += 2;
				}

				if (potentialTarget.isHero() || potentialTarget.isUnit()) {
					if (potentialTarget.canReach(su,
							getMouvementObstacles(potentialTarget))) {
						poid += 2;
					}
				}

				return poid;
			}

		};

	}

	/**
	 * do validate new actor-target couple for this phase
	 * 
	 * @param actor
	 *            the actor
	 * @param target
	 *            the target
	 */
	public void doAddActorAndTarget(Entity actor, Entity target) {
		super.doAddActorAndTarget(actor, target);
		((Shootingunit) actor).setTarget(target);
	}

	/**
	 * set phase actor
	 * 
	 * @param actor
	 *            the actor
	 */
	public void setActor(Entity actor) {
		Shootingunit oldActor = (Shootingunit) getActor();
		Shootingunit newActor = (Shootingunit) actor;
		super.setActor(newActor);
		if (oldActor != null) {
			BattleFieldPanel.refreshIt(oldActor.getShootingRangeBounds());
		}
		if (newActor != null) {
			BattleFieldPanel.refreshIt(newActor.getShootingRangeBounds());
		}

	}

	/**
	 * switch actor selection mode
	 * 
	 * @param actorselection
	 *            actor selection or target selection ?
	 */
	public void setActorselection(boolean actorselection) {
		Shootingunit actor = (Shootingunit) getActor();
		if (actor != null) {
			BattleFieldPanel.refreshIt(actor.getShootingRangeBounds());
		}
		super.setActorselection(actorselection);
	}

}
