/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.player;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

/**
 * a player table model
 * 
 * @author farfpeuf
 * 
 */
public class PlayersTableModel extends AbstractTableModel {
	/**
	 * the players list
	 */
	private List playerList = new ArrayList();

	/**
	 * is this table editable ?
	 */
	private boolean editable;

	/**
	 * constructor
	 * 
	 * @param editable
	 *            is this table editable ?
	 */
	public PlayersTableModel(boolean editable) {
		this.editable = editable;
	}

	/**
	 * add a player to table
	 * 
	 * @param name
	 *            player name
	 * @param army
	 *            player army
	 * @param isAi
	 *            human player or artificial intelligence?
	 * @param side
	 *            player side
	 */
	public void addPlayer(String name, String army, boolean isAi, int side) {

		Player pla = new Player(army, isAi, name, side);
		playerList.add(pla);
		fireTableRowsInserted(playerList.size() - 1, playerList.size() - 1);
	}

	/**
	 * remove player from table
	 * 
	 * @param row
	 *            the player row
	 */
	public void removePlayer(int row) {
		playerList.remove(row);
		fireTableRowsDeleted(row, row);
	}

	/**
	 * get player at specified row
	 * 
	 * @param row
	 *            the row
	 * @return the player
	 */
	public Player getPla(int row) {
		return (Player) playerList.get(row);
	}

	/**
	 * change player side
	 * 
	 * @param row
	 *            the player row
	 * @param side
	 *            the new side
	 */
	public void setPlayerSide(int row, int side) {
		Player pla = (Player) playerList.get(row);
		if (pla != null) {
			pla.setSide(side);
		}
		fireTableCellUpdated(row, 3);
	}

	/**
	 * change player name
	 * 
	 * @param row
	 *            the player row
	 * @param name
	 *            the new name
	 */
	public void setPlayerName(int row, String name) {
		Player pla = (Player) playerList.get(row);
		if (pla != null) {
			pla.setName(name);
		}
		fireTableCellUpdated(row, 0);
	}

	/**
	 * change player army
	 * 
	 * @param row
	 *            the player row
	 * @param name
	 *            the new army
	 */
	public void setPlayerArmy(int row, String army) {
		Player pla = (Player) playerList.get(row);
		if (pla != null) {
			pla.setArmyName(army);
		}
		fireTableCellUpdated(row, 2);

	}

	/**
	 * change player ai attribute
	 * 
	 * @param row
	 *            the player row
	 * @param name
	 *            the new ai attribute
	 */
	public void setPlayerAI(int row, boolean ai) {
		Player pla = (Player) playerList.get(row);
		if (pla != null) {
			pla.setIa(ai);
		}
		fireTableCellUpdated(row, 1);
	}

	/**
	 * @return number of players
	 */
	public int getNbPlayer() {
		return playerList.size();
	}

	/**
	 * @return the number of columns.
	 */
	public int getColumnCount() {
		return 4;
	}

	/**
	 * @return the number of rows.
	 */
	public int getRowCount() {
		return playerList.size();
	}

	/**
	 * Returns the value for the cell at <code>columnIndex</code> and
	 * <code>rowIndex</code>.
	 * 
	 * @param rowIndex
	 *            the row whose value is to be queried
	 * @param columnIndex
	 *            the column whose value is to be queried
	 * @return the value Object at the specified cell
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		Player pla = (Player) playerList.get(rowIndex);
		switch (columnIndex) {
		case 0:
			return pla.getName();
		case 1:
			return new Boolean(pla.isIa());
		case 2:
			return pla.getArmyName();
		case 3:
			return "" + (char) ('A' + pla.getSide());
		}
		return null;
	}

	/**
	 * Sets the value in the cell at <code>columnIndex</code> and
	 * <code>rowIndex</code> to <code>aValue</code>.
	 * 
	 * @param aValue
	 *            the new value
	 * @param rowIndex
	 *            the row whose value is to be changed
	 * @param columnIndex
	 *            the column whose value is to be changed
	 * @see #getValueAt
	 * @see #isCellEditable
	 */
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		Player pla = (Player) playerList.get(rowIndex);
		switch (columnIndex) {
		case 0:
			pla.setName((String) value);
			break;
		case 1:
			pla.setIa(((Boolean) value).booleanValue());
			break;
		case 2:
			pla.setArmyName((String) value);
			break;
		case 3:
			String side = (String) value;
			pla.setSide(side.charAt(0) - 'A');
			break;
		}
	}

	/**
	 * Returns true if the cell at <code>rowIndex</code> and
	 * <code>columnIndex</code> is editable. Otherwise,
	 * <code>setValueAt</code> on the cell will not change the value of that
	 * cell.
	 * 
	 * @param rowIndex
	 *            the row whose value to be queried
	 * @param columnIndex
	 *            the column whose value to be queried
	 * @return true if the cell is editable
	 * @see #setValueAt
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return editable;
	}

}
