/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.sound;

import java.util.LinkedList;

import javax.sound.sampled.Clip;

/**
 * A list of all the available sounds
 * 
 * @author farfpeuf
 * 
 */

class SoundList extends java.util.Hashtable {

	public int nbready;

	/**
	 * Constructs an empty soundlist and initializes the Hashtable with a
	 * capacity of 32 entries.
	 */

	public SoundList() {
		super(32);
	}

	/**
	 * gets an avaible Clip of the specified type of sound
	 * 
	 * @param soundType
	 *            the type of sound to get.
	 * 
	 * @return the next available clip for this type of sound or null if none is
	 *         available.
	 */

	public Clip getClip(String soundType) {

		// try to find the clip list for this type of sound

		LinkedList clipList = (LinkedList) get(soundType);

		// return the first clip of the list or null if the list is empty

		if (clipList == null) {
			return null;
		}

		try {
			return (Clip) clipList.removeFirst();
		} catch (java.util.NoSuchElementException nsee) {
			return null;
		}
	}

	/**
	 * makes available a newly loaded Clip of the specified type of sound.
	 * 
	 * @param clip
	 *            the clip that is now available.
	 * @param soundType
	 *            the type of sound of the clip.
	 */

	public void putClip(Clip clip, String soundType) {

		// try to find the clip list for this type of sound or create it if
		// there is none

		LinkedList clipList = (LinkedList) get(soundType);

		if (clipList == null) {
			clipList = new LinkedList();
			put(soundType, clipList);
		}

		// add the clip to the clip list

		clipList.addLast(clip);
		nbready++;

	}// putClip

}
