/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.spells;

import javax.swing.ImageIcon;

import fb.army.entities.Entity;
import fb.army.entities.Leader;
import fb.army.entities.SpellCaster;
import fb.battlefieldelements.MagicEffect;
import fb.util.D6;
import fb.util.Resources;

/**
 * transmit an order to a friendly battle leader
 * 
 * @author farfpeuf
 * 
 */
public class Telepathy extends TargetSpell {
	/**
	 * the new order
	 */
	private char neworder = 'a';

	/**
	 * spell visual effects
	 */
	private static final ImageIcon[] effects = new ImageIcon[] { Resources
			.getSign("transmissionmagic") };

	/**
	 * constructor
	 * 
	 */
	public Telepathy() {
		super();
		name = "telepathy";
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            spell actor
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity a, Entity e) {
		if (e.isLeader() && !e.isEnnemy(a) && super.isTarget(a, e)) {
			Leader l = (Leader) e;
			if (l.orderDistanceOkForAll() && !l.isWarchief()) {
				return true;
			}
		}
		return false;
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for current spell
	 *         caster
	 */
	public boolean isTarget(Entity e) {
		if (caster.nearWarchief() && e.isLeader() && !e.isEnnemy(caster)
				&& super.isTarget(e)) {
			Leader l = (Leader) e;
			if (!l.isWarchief() && l.orderDistanceOkForAll()) {
				return true;
			}
		}
		return false;
	}

	/**
	 * cast the spell
	 * 
	 * @return true if the spell was successfull
	 */
	public boolean castSpell() {
		if (caster.spendMagicPoints(cost())) {
			return true;
		}

		return false;

	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell cost
	 */
	public int cost(SpellCaster caster, Entity target) {
		return D6.roll() + (int) (caster.stepsTo(target) / 12);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell maximum cost
	 */
	public int maxCost(SpellCaster caster, Entity target) {
		return 6 + (int) (caster.stepsTo(target) / 12);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell minimumu cost
	 */
	public int minCost(SpellCaster caster, Entity target) {
		return 1 + (int) (caster.stepsTo(target) / 12);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return average cost
	 */
	public int averageCost(SpellCaster caster, Entity target) {
		return (int) (3.5 + (caster.stepsTo(target) / 12));
	}

	/**
	 * prepare spell visual effects
	 * 
	 */
	public void prepareEffects() {
		caster.getMagics().add(
				new MagicEffect(caster, ((Leader) target).getModel(), effects));
	}

	/**
	 * end the spell
	 * 
	 * @throws InterruptedException
	 */
	public void endSpell() throws InterruptedException {
		if (caster.isHandled()) {
			((Leader) target).magicNewOrder(neworder);
		}
		magicSynchronize();
	}

	/**
	 * @return Returns the neworder.
	 */
	public char getNeworder() {
		return neworder;
	}

	/**
	 * @param neworder
	 *            The neworder to set.
	 */
	public void setNeworder(char neworder) {
		this.neworder = neworder;
	}

	/**
	 * @return true if AI player should cast this spell
	 */
	public boolean shouldAICast() {
		return false;// TODO
	}

}
