/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.util;

import javax.swing.ListModel;
import javax.swing.MutableComboBoxModel;
import javax.swing.event.ListDataListener;

/**
 * list model and MutableComboBoxModel using arraylistmodel as delegate
 * 
 * @author farfpeuf
 * 
 */
public class ComboArrayListModel implements ListModel, MutableComboBoxModel {

	/**
	 * the delegate list that contains the data
	 */
	protected ArrayListModel delegate;

	protected Object selectedObject;

	/**
	 * Default constructor. Uses an ArrayList for implementation.
	 * 
	 */
	public ComboArrayListModel() {
		this(new ArrayListModel());
	}

	/**
	 * Uses an ArrayListModel for implementation.
	 * 
	 * @param collection
	 *            the collection whose elements are to be placed into this model
	 */
	public ComboArrayListModel(ArrayListModel delegate) {
		super();
		this.delegate = delegate;
	}

	/**
	 * @see javax.swing.MutableComboBoxModel#addElement(java.lang.Object)
	 */
	public void addElement(Object obj) {
		delegate.add(obj);
		if (delegate.size() == 1 && selectedObject == null && obj != null) {
			setSelectedItem(obj);
		}

	}

	/**
	 * @see javax.swing.MutableComboBoxModel#insertElementAt(java.lang.Object,
	 *      int)
	 */
	public void insertElementAt(Object obj, int index) {
		delegate.add(index, obj);

	}

	/**
	 * @see javax.swing.MutableComboBoxModel#removeElement(java.lang.Object)
	 */
	public void removeElement(Object obj) {
		int index = delegate.indexOf(obj);
		if (index != -1) {
			removeElementAt(index);
		}
	}

	/**
	 * @see javax.swing.MutableComboBoxModel#removeElementAt(int)
	 */
	public void removeElementAt(int index) {
		if (getElementAt(index) == selectedObject) {
			if (index == 0) {
				setSelectedItem(getSize() == 1 ? null : getElementAt(index + 1));
			} else {
				setSelectedItem(getElementAt(index - 1));
			}
		}
		delegate.remove(index);

	}

	/**
	 * @see javax.swing.ComboBoxModel#getSelectedItem()
	 */
	public Object getSelectedItem() {
		return selectedObject;
	}

	/**
	 * @see javax.swing.ComboBoxModel#setSelectedItem(java.lang.Object)
	 */
	public void setSelectedItem(Object anItem) {
		if ((selectedObject != null && !selectedObject.equals(anItem))
				|| selectedObject == null && anItem != null) {
			selectedObject = anItem;
			delegate.fireContentsChanged(this, -1, -1);
		}
	}

	/**
	 * @see javax.swing.ListModel#addListDataListener(javax.swing.event.ListDataListener)
	 */
	public void addListDataListener(ListDataListener l) {
		delegate.addListDataListener(l);

	}

	/**
	 * @see javax.swing.ListModel#getElementAt(int)
	 */
	public Object getElementAt(int index) {
		return delegate.getElementAt(index);
	}

	/**
	 * @see javax.swing.ListModel#getSize()
	 */
	public int getSize() {
		return delegate.getSize();
	}

	/**
	 * @see javax.swing.ListModel#removeListDataListener(javax.swing.event.ListDataListener)
	 */
	public void removeListDataListener(ListDataListener l) {
		delegate.removeListDataListener(l);
	}

	/**
	 * @return the available elements list
	 */
	public ArrayListModel getAvailableElements() {
		return delegate;
	}

}