/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.spec.AlgorithmParameterSpec;

import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

/**
 * utility class to encode and decode bytes streams
 * 
 * @author farfpeuf
 * 
 */
public class Crypto {
	/**
	 * encryption cipher
	 */
	private Cipher ecipher;

	/**
	 * decryption cipher
	 */
	private Cipher dcipher;

	/**
	 * singleton instance
	 */
	private static Crypto instance;

	/**
	 * decoded encryption key
	 */
	private static final byte[] decodedKeyC = new byte[] { (byte) 0x22,
			(byte) 0x14, (byte) 0x10, (byte) 0x12, (byte) 0x13, (byte) 0x16,
			(byte) 0x18, (byte) 0xB5 };

	/**
	 * bytes buffer
	 */
	private byte[] buf = new byte[1024];

	/**
	 * encode a file
	 * 
	 * @param file
	 *            the file to encode
	 * @throws Exception
	 */
	public static void encode(File file) throws Exception {
		if (file.exists()) {
			System.out.println("Encoding " + file.getAbsolutePath());
			if (file.isDirectory()) {
				File[] subfiles = file.listFiles();
				for (int i = 0; i < subfiles.length; i++) {
					encode(subfiles[i]);
				}
			} else {
				if (!file.getAbsolutePath().endsWith("Thumbs.db")) {
					getInstance().encrypt(
							new FileInputStream(file),
							new FileOutputStream(FileTools.getEncodedPath(file
									.getAbsolutePath())));
				}
				file.delete();
			}
		}
	}

	/**
	 * @return encryption key
	 * @throws Exception
	 */
	private static SecretKey getKey() throws Exception {
		SecretKeySpec desKeySpec = new SecretKeySpec(decodedKeyC, 0, 8, "DES");
		SecretKey key = (SecretKey) desKeySpec;
		return key;
	}

	/**
	 * constructor
	 * 
	 */
	private Crypto() {
		try {
			SecretKey key = getKey();
			byte[] iv = new byte[] { (byte) 0x8E, 0x12, 0x39, (byte) 0x9C,
					0x07, 0x72, 0x6F, 0x5A };
			AlgorithmParameterSpec paramSpec = new IvParameterSpec(iv);
			ecipher = Cipher.getInstance("DES/CBC/PKCS5Padding");
			dcipher = Cipher.getInstance("DES/CBC/PKCS5Padding");
			ecipher.init(Cipher.ENCRYPT_MODE, key, paramSpec);
			dcipher.init(Cipher.DECRYPT_MODE, key, paramSpec);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * encrypt a stream
	 * 
	 * @param in
	 *            input
	 * @param out
	 *            output
	 */
	public void encrypt(InputStream in, OutputStream out) {
		try {
			out = new CipherOutputStream(out, ecipher);
			int numRead = 0;
			while ((numRead = in.read(buf)) >= 0) {
				out.write(buf, 0, numRead);
			}
			in.close();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * decrypt a stream
	 * 
	 * @param in
	 *            input
	 * @param out
	 *            output
	 */
	public void decrypt(InputStream in, OutputStream out) {
		try {
			in = new CipherInputStream(in, dcipher);
			int numRead = 0;
			while ((numRead = in.read(buf)) >= 0) {
				out.write(buf, 0, numRead);
			}
			in.close();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static synchronized Crypto getInstance() {
		if (instance == null) {
			instance = new Crypto();
		}
		return instance;
	}
}