/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.util;

/**
 * utility class for string manipulations
 * 
 * @author Farfpeuf
 */
public class StringTools {

	/**
	 * default token separator
	 */
	public static final char defaultSeparator = '\t';

	/**
	 * Replaces each substring of this source string that matches the given
	 * string to replace with the given replacement.
	 * 
	 * @param source
	 *            the source string
	 * @param toReplace
	 *            the string to replace (not a regular expression)
	 * @param replaceBy
	 *            the replacement string
	 * @return the modified string
	 */

	public static String replaceAll(String source, String toReplace,
			String replaceBy) {

		String result = "", reste = source;
		int toReplLength = toReplace.length();
		int i = reste.indexOf(toReplace);
		while (i != -1) {
			result += reste.substring(0, i) + replaceBy;
			reste = reste.substring(i + toReplLength);
			i = reste.indexOf(toReplace);
		}
		result += reste;
		return result;
	}

	/**
	 * read next token in a string buffer
	 * 
	 * @param buffy
	 *            the string buffer
	 * @param separator
	 *            the token separator
	 * @return the next token
	 */
	public static String readBuffer(StringBuffer buffy, char separator) {
		String result;
		int index = buffy.toString().indexOf(separator);
		if (index != -1) {
			result = buffy.substring(0, index);
			buffy = buffy.delete(0, index + 1);
		} else {
			result = buffy.toString();
			buffy = buffy.delete(0, buffy.length());
		}
		return result;
	}

	/**
	 * read next token in a string buffer
	 * 
	 * @param buffy
	 *            the string buffer
	 * @return the next token
	 */
	public static String readBuffer(StringBuffer buffy) {
		return readBuffer(buffy, defaultSeparator);
	}

	/**
	 * read next token in a string buffer and parse it as an integer
	 * 
	 * @param buffy
	 *            the string buffer
	 * @return the parsed integer
	 */
	public static int readBufferInt(StringBuffer buffy) {
		return Integer.parseInt(readBuffer(buffy, defaultSeparator));
	}

	/**
	 * read next token in a string buffer and parse it as an float
	 * 
	 * @param buffy
	 *            the string buffer
	 * @return the parsed float
	 */
	public static float readBufferFloat(StringBuffer buffy) {
		return Float.parseFloat(readBuffer(buffy, defaultSeparator));
	}

	/**
	 * read next token in a string buffer and parse it as a double
	 * 
	 * @param buffy
	 *            the string buffer
	 * @return the parsed double
	 */
	public static double readBufferDouble(StringBuffer buffy) {
		return Double.parseDouble(readBuffer(buffy, defaultSeparator));
	}

	/**
	 * read next token in a string buffer and parse it as a char
	 * 
	 * @param buffy
	 *            the string buffer
	 * @return the parsed char
	 */
	public static char readBufferChar(StringBuffer buffy) {
		return readBuffer(buffy, defaultSeparator).charAt(0);
	}

	/**
	 * read next token in a string buffer and parse it as an integer
	 * 
	 * @param buffy
	 *            the string buffer
	 * @param separator
	 *            the token separator
	 * @return the parsed integer
	 */
	public static int readBufferInt(StringBuffer buffy, char separator) {
		return Integer.parseInt(readBuffer(buffy, separator));
	}

	/**
	 * read next token in a string buffer and parse it as a float
	 * 
	 * @param buffy
	 *            the string buffer
	 * @param separator
	 *            the token separator
	 * @return the parsed float
	 */
	public static float readBufferFloat(StringBuffer buffy, char separator) {
		return Float.parseFloat(readBuffer(buffy, separator));
	}

	/**
	 * read next token in a string buffer and parse it as a char
	 * 
	 * @param buffy
	 *            the string buffer
	 * @param separator
	 *            the token separator
	 * @return the parsed char
	 */
	public static char readBufferChar(StringBuffer buffy, char separator) {
		return readBuffer(buffy, separator).charAt(0);
	}

	/**
	 * read next token in a string buffer and parse it as a boolean
	 * 
	 * @param buffy
	 *            the string buffer
	 * @param separator
	 *            the token separator
	 * @return the parsed boolean
	 */
	public static boolean readBufferBoolean(StringBuffer buffy, char separator) {
		return Boolean.valueOf(readBuffer(buffy, separator)).booleanValue();
	}

	/**
	 * read next token in a string buffer and parse it as a double
	 * 
	 * @param buffy
	 *            the string buffer
	 * @param separator
	 *            the token separator
	 * @return the parsed double
	 */
	public static double readBufferDouble(StringBuffer buffy, char separator) {
		return Double.parseDouble(readBuffer(buffy, separator));
	}

}
